@@ -18,9 +18,11 @@
 // http://www.swarm.org/
 
 #include "nsISupports.h"
+#include "nsISupportsPrimitives.h"
 #include "nsIInterfaceInfo.h"
 #include "nsIInterfaceInfoManager.h"
 #include "nsIComponentManager.h"
+#include "nsIComponentManagerObsolete.h"
 #include "nsIServiceManager.h"
 #include "nsIXPConnect.h"
 #include "nsIEnumerator.h"
@@ -57,21 +59,15 @@ struct collect_methods_t {
 };
   
 
-static void *
-find (void *(*match) (nsIInterfaceInfo *, void *), void *item)
+static const void *
+find (const void *(*match) (nsIInterfaceInfo *, void *), void *item)
 {
-  nsIInterfaceInfoManager *iim = nsnull;
+  nsCOMPtr <nsIInterfaceInfoManager> iim (do_GetService (NS_INTERFACEINFOMANAGER_SERVICE_CONTRACTID));
   nsIEnumerator *Interfaces = nsnull;
   nsISupports *is_Interface;
   nsIInterfaceInfo *Interface;
   nsresult rv;
-  void *ret = NULL;
-  
-  if (!(iim = XPTI_GetInterfaceInfoManager ()))
-    {
-      NS_ASSERTION (0, "failed to get the InterfaceInfoManager");
-      goto done;
-    }
+  const void *ret = NULL;
   
   if (NS_FAILED (iim->EnumerateInterfaces (&Interfaces)))
     {
@@ -87,8 +83,7 @@ find (void *(*match) (nsIInterfaceInfo *, void *), void *item)
   
   do
     {
-      if (NS_FAILED (rv = Interfaces->CurrentItem (&is_Interface)))
-        {
+      if (NS_FAILED (rv = Interfaces->CurrentItem (&is_Interface))) {
           /* maybe something should be done,
            * debugging info at least? */
           Interfaces->Next ();
@@ -100,21 +95,18 @@ find (void *(*match) (nsIInterfaceInfo *, void *), void *item)
                         (void **) &Interface);
       
       if (!NS_FAILED (rv))
-        {
-          ret = (*match) (Interface, item);
-          if (ret != Interface)
-            NS_RELEASE (Interface);
-        }
+	ret = (*match) (Interface, item);
       else
         abort ();
       
       if (ret)
         break;
 
+      NS_RELEASE (Interface);
       NS_RELEASE (is_Interface);
       Interfaces->Next ();
       
-    } while (NS_COMFALSE == Interfaces->IsDone ());
+    } while (Interfaces->IsDone () == NS_ENUMERATOR_FALSE);
   
  done:
   NS_IF_RELEASE (Interfaces);
@@ -124,7 +116,7 @@ find (void *(*match) (nsIInterfaceInfo *, void *), void *item)
   return ret;
 }
 
-static void *
+static const void *
 matchInterfaceName (nsIInterfaceInfo *interfaceInfo, void *item)
 {
   nsIID *ret = NULL;
@@ -134,27 +126,30 @@ matchInterfaceName (nsIInterfaceInfo *interfaceInfo, void *item)
   interfaceInfo->GetName (&name);
   
   if (PL_strcmp (interfaceName, name) == 0)
-    interfaceInfo->GetIID (&ret);
+    interfaceInfo->GetInterfaceIID (&ret);
   nsMemory::Free (name);
   return ret;
 }
 
-static nsIID *
+static const nsIID *
 findIIDFromName (const char *interfaceName)
 {
-  return (nsIID *) find (matchInterfaceName, (void *) interfaceName);
+  return (const nsIID *) find (matchInterfaceName, (void *) interfaceName);
 }
 
-static void *
+const static void *
 matchMethodName (nsIInterfaceInfo *interfaceInfo, void *item)
 {
   struct method_pair *pair = (struct method_pair *) item;
   nsISupports *interface;
   nsIID *iid;
 
-  if (interfaceInfo->GetIID (&iid) != NS_OK)
+  if (interfaceInfo->GetInterfaceIID (&iid) != NS_OK)
     abort ();
 
+char *foo;
+interfaceInfo->GetName (&foo);
+
   if (pair->key.target->QueryInterface (*iid, (void **) &interface) == NS_OK)
     {
       if (NS_SUCCEEDED (interfaceInfo->GetMethodInfoForName
@@ -170,28 +165,48 @@ matchMethodName (nsIInterfaceInfo *interfaceInfo, void *item)
 return NULL;
 }
 
+static const nsIID
+getSwarmIID (const char *interfaceName)
+{
+  nsIID iid;
+  nsIID default_iid = NS_GET_IID (nsISupports);
+
+  if (interfaceName)
+    {
+      char *buf = (char *) malloc (6 + PL_strlen (interfaceName) + 1);
+      const nsIID *iidptr;
+
+      if (buf == NULL)
+        abort ();
+      PL_strcpy (buf, "swarmI");
+      PL_strcat (buf, interfaceName);
+      
+      if ((iidptr = findIIDFromName (buf)))
+        iid = *iidptr;
+      else
+        iid = default_iid;
+      free (buf);
+    }
+  else
+    iid = default_iid;
+
+  return iid;
+}
+
 static nsISupports *
 createComponentByName (const char *contractID, const char *interfaceName)
 {
   nsISupports *obj;
   nsresult rv;
-  nsIID *iid;
-  nsIID default_iid = NS_GET_IID (nsISupports);
+  nsIID iid;
 
-  if (interfaceName)
-    {
-      char buf[6 + PL_strlen (interfaceName) + 1];
-
-      PL_strcpy (buf, "swarmI");
-      PL_strcat (buf, interfaceName);
-      
-      if (!(iid = findIIDFromName (buf)))
-        iid = &default_iid;
-    }
-  else
-    iid = &default_iid;
-  
-  rv = nsComponentManager::CreateInstance (contractID, NULL, *iid, (void **) &obj);
+ 
+  nsCOMPtr<nsIComponentManager> compMgr;
+  NS_GetComponentManager (getter_AddRefs (compMgr));  
+  if (!compMgr)
+    abort (); 
+  iid = getSwarmIID (interfaceName);
+  rv = compMgr->CreateInstanceByContractID (contractID, NULL, iid, (void **) &obj);
   if (NS_FAILED (rv))
     abort ();
   return obj;
@@ -206,7 +221,9 @@ COMfindComponent (const char *className)
   size_t modulePrefixLen = PL_strlen (modulePrefix);
   nsCID *cClass = new nsCID ();
   size_t classNameLen = PL_strlen (className);
-  char buf[prefixLen + classNameLen + 1];
+  char *buf = (char *) malloc (prefixLen + classNameLen + 1);
+  if (!buf)
+    abort ();
   nsresult rv;
 
   PL_strcpy (buf, prefix);
@@ -224,9 +241,14 @@ COMfindComponent (const char *className)
             buf[pos] = '.';
         }
     }
-
-  rv = nsComponentManager::ContractIDToClassID (buf, cClass);
-
+  nsCOMPtr<nsIComponentManager> compMgr;
+  NS_GetComponentManager (getter_AddRefs (compMgr));  
+  if (!compMgr)
+    abort ();
+  nsCOMPtr<nsIComponentManagerObsolete> compMgrO = do_QueryInterface (compMgr);
+  rv = compMgrO->ContractIDToClassID (buf, cClass);
+  free (buf);
+ 
   if (NS_FAILED (rv))
     abort ();
   return (COMclass) cClass;
@@ -235,17 +257,17 @@ COMfindComponent (const char *className)
 COMobject
 COMcreateComponent (COMclass cClass)
 {
-  nsCID *cid = (nsCID *) cClass;
-  char *className;
-  char *contractID;
+  const nsCID *cid = (const nsCID *) cClass;
+  nsIID iid;
+  const char *className = COM_get_class_name (cClass);
   char *interfaceName = NULL;
-  nsresult rv = nsComponentManager::CLSIDToContractID (cid, &className, &contractID);
+  nsCOMPtr<nsIComponentManager> compMgr;
+  NS_GetComponentManager (getter_AddRefs (compMgr));  
+  if (!compMgr)
+    abort ();
   size_t len;
   nsISupports *obj;
 
-  if (NS_FAILED (rv))
-    abort ();
-  
   len = PL_strlen (className);
 
   if (len > 10) /* swarm + name + Impl */
@@ -260,9 +282,14 @@ COMcreateComponent (COMclass cClass)
     }
   else
     interfaceName = NULL;
-
-  obj = createComponentByName (contractID, interfaceName);
   
+  iid = getSwarmIID (interfaceName);
+  if (!NS_SUCCEEDED (compMgr->CreateInstance (*cid, nsnull, iid, (void **) &obj)))
+    obj = nsnull;
+
+  if (obj == nsnull)
+    abort ();
+
   if (interfaceName)
     PL_strfree (interfaceName);
 
@@ -337,12 +364,33 @@ COMgetClass (COMobject cObj)
 const char *
 COMgetComponentName (COMclass cClass)
 {
-  char *name, *contractID;
+  char *name = NULL;
   nsCID *cid = (nsCID *) cClass;
+  nsCOMPtr<nsIComponentManager> compMgr;
+  nsIEnumerator *contractEnum;
+  nsISupportsCString *cval;
+  nsCID mcid;
 
-  if (NS_FAILED (nsComponentManager::CLSIDToContractID (cid, &name, &contractID)))
+  NS_GetComponentManager (getter_AddRefs (compMgr));  
+  if (!compMgr)
     abort ();
+  nsCOMPtr<nsIComponentManagerObsolete> compMgrO = do_QueryInterface (compMgr);
 
+  compMgrO->EnumerateContractIDs (&contractEnum);
+  contractEnum->First ();
+
+  while (contractEnum->IsDone () == NS_ENUMERATOR_FALSE) {
+    contractEnum->CurrentItem ((nsISupports **) &cval);
+    cval->ToString (&name);
+    compMgrO->ContractIDToClassID (name, &mcid);
+    if (cid->Equals (mcid))
+      break; 
+    else if (name) {
+      nsMemory::Free (name);
+      name = NULL;
+    }
+    contractEnum->Next ();
+  }
   return name;
 }
 
@@ -381,10 +429,13 @@ COMselector
 selectorCreate (COMmethod cMethod)
 {
   swarmISelector *cSel, *ret;
-
-  if (!NS_SUCCEEDED (nsComponentManager::CreateInstance (kSelector, nsnull,
-                                                         NS_GET_IID (swarmISelector),
-                                                         (void **) &cSel)))
+  nsCOMPtr<nsIComponentManager> compMgr;
+  NS_GetComponentManager (getter_AddRefs (compMgr));  
+  if (!compMgr)
+    abort ();
+  if (!NS_SUCCEEDED (compMgr->CreateInstance (kSelector, nsnull,
+                                              NS_GET_IID (swarmISelector),
+                                              (void **) &cSel)))
     abort ();
   if (!NS_SUCCEEDED (cSel->CreateFromMethod (cMethod, &ret)))
     abort ();
@@ -520,7 +571,7 @@ destroyMethod (nsHashKey *key, void *data, void *param)
   return PR_TRUE;
 }
 
-static void *
+const static void *
 matchImplementedInterfaces (nsIInterfaceInfo *interfaceInfo, void *item)
 {
   struct collect_methods_t *info = (struct collect_methods_t *) item;
@@ -530,7 +581,7 @@ matchImplementedInterfaces (nsIInterfaceInfo *interfaceInfo, void *item)
   if (info->collectVariableFunc && info->collectMethodFunc)
     abort ();
 
-  if (!NS_SUCCEEDED (interfaceInfo->GetIID (&iid)))
+  if (!NS_SUCCEEDED (interfaceInfo->GetInterfaceIID (&iid)))
     abort ();
 
   nsISupports *interface;
@@ -557,7 +608,7 @@ matchImplementedInterfaces (nsIInterfaceInfo *interfaceInfo, void *item)
           
           struct method_value *method = new method_value;
           
-          if (!NS_SUCCEEDED (interfaceInfo->GetIID ((nsIID **) &method->methodIID)))
+          if (!NS_SUCCEEDED (interfaceInfo->GetInterfaceIID ((nsIID **) &method->methodIID)))
             abort ();
           method->methodIndex = i;
           method->methodInfo = methodInfo;
@@ -822,7 +873,7 @@ COMmethodInvoke (COMmethod cMethod, COMobject obj, void *params)
                                            (void **) &interface)))
     abort ();
 
-  if (!NS_SUCCEEDED (XPTC_InvokeByIndex (interface,
+  if (!NS_SUCCEEDED (NS_InvokeByIndex (interface,
                                          method->methodIndex,
                                          method->methodInfo->GetParamCount (),
                                          (nsXPTCVariant *) params)))
