@@ -35,6 +35,8 @@
   ("swarmAveragerImpl" . "fa3b7641-b884-4073-a6b4-be871378d206")
   ("swarmBernoulliDistCImpl" . "107e5636-b3a0-4bcc-88d8-afbdf1e4a189")
   ("swarmBernoulliDistImpl" . "39ae98a5-8a32-4d57-a348-0267b5452f2f")
+  ("swarmBinomialDistCImpl" . "aba07003-a5e0-4f36-8696-40bfac3ea46e")
+  ("swarmBinomialDistImpl" . "a417b554-8f2d-45f6-acf7-3b9ca493b51f")
   ("swarmC2TAUS1genCImpl" . "2b533a1e-2867-4d11-a350-b5727dc55b47")
   ("swarmC2TAUS1genImpl" . "7f1d62f1-f1c2-4d5f-ac59-5e99449440e2")
   ("swarmC2TAUS2genCImpl" . "581820e2-d8fe-4542-8286-96d52c237439")
@@ -212,6 +214,9 @@
   ("swarmIBernoulliDist" . "bb7e6b11-abb5-482a-be50-6b7f518354fd")
   ("swarmIBernoulliDistC" . "65f766eb-ec2e-46ce-a415-e1d2d93b93e4")
   ("swarmIBernoulliDistS" . "0c0de570-04b3-4bb9-b6a5-f5b25261d234")
+  ("swarmIBinomialDist" . "80905362-527b-4d40-ad3c-a80aad4837a5")
+  ("swarmIBinomialDistC" . "bd530509-56a1-437d-adb0-05511bc2a255")
+  ("swarmIBinomialDistS" . "ca8c0bb6-248b-4b7c-bcf5-88386c9a5822")
   ("swarmIBooleanDistribution" . "5056fc94-8095-48e2-ae09-28d0f6f3f8e1")
   ("swarmIBooleanDistributionC" . "68212f78-d5ee-4b37-a996-d9b791c9bcf2")
   ("swarmIBooleanDistributionS" . "866e52b8-74b2-4433-969f-d546b695c79c")
