@@ -11,17 +11,16 @@ DIE=0
 NOCONFIGURE=1
 
 # Hack here for autohell - substitute your install location or nothing for default path
-# ACDIR=/usr/local/autoconf-2.57/bin/
-LTDIR=/usr/local/libtool-1.5a/bin/
+# LTDIR=/usr/local/libtool-2.2/bin
 # AMDIR=/usr/local/automake-1.7.5/bin/
 ACDIR=
-# LTDIR=
+LTDIR=
 AMDIR=
 
 # Update whenever version dependencies of developer tools change
-REQUIRED_AUTOCONF_VERSION="2.57"
-#REQUIRED_LIBTOOL_VERSION="1.5a"
-REQUIRED_AUTOMAKE_VERSION="1.6.3"
+REQUIRED_AUTOCONF_VERSION="2.62"
+REQUIRED_LIBTOOL_VERSION="2.2.6b"
+REQUIRED_AUTOMAKE_VERSION="1.10.1"
 
 ("$ACDIR"autoconf --version) < /dev/null > /dev/null 2>&1 || {
   echo
@@ -46,6 +45,7 @@ fi
 #	  }
 	
 #Why do we care about the system libtool?  Everything needed is in the source distribution.
+#A. If we need some libtool macro in aclocal.m4 and we are not using the system libtool.
 #	LIBTOOL_VERSION=`("$LTDIR"libtool --version) | head -1|cut -d')' -f2| cut -d'(' -f1|sed  's/ //g'`
 #else
 #	("$LTDIR"libtool --version) < /dev/null > /dev/null 2>&1 || {
@@ -54,13 +54,13 @@ fi
 #		DIE=1
 #	  }
 	
-#	LIBTOOL_VERSION=`("$LTDIR"libtool --version) | head -1|cut -d')' -f2| cut -d'(' -f1|sed  's/ //g'`
-#fi
-#if [ "$LIBTOOL_VERSION" != "$REQUIRED_LIBTOOL_VERSION" ]; then
-#	echo
-#	echo "**Warning**: only tested with version" $REQUIRED_LIBTOOL_VERSION
-#	echo "of libtool and may not work with version" $LIBTOOL_VERSION
+LIBTOOL_VERSION=`("$LTDIR"libtool --version) | head -1|cut -d')' -f2| cut -d'(' -f1|sed  's/ //g'`
 #fi
+if [ "$LIBTOOL_VERSION" != "$REQUIRED_LIBTOOL_VERSION" ]; then
+	echo
+	echo "**Warning**: only tested with version" $REQUIRED_LIBTOOL_VERSION
+	echo "of libtool and may not work with version" $LIBTOOL_VERSION
+fi
 
 ("$AMDIR"automake --version) < /dev/null > /dev/null 2>&1 || {
   echo
@@ -125,8 +125,8 @@ do
       echo "Running autoconf ..."
       "$ACDIR"autoconf
       if test -f ./Makefile.am; then
-	  echo "Running automake --gnu $am_opt ..."
-	  "$AMDIR"automake --add-missing --gnu $am_opt
+	  echo "Running automake --gnu -Wno-portability $am_opt ..."
+	  "$AMDIR"automake --add-missing --gnu -Wno-portability $am_opt
       else
 	  echo skipping "automake" in $dr -- Makefile.am does not exist
       fi
