@@ -25,13 +25,25 @@
 #import <random.h>
 #import <objectbase.h>
 #import <simtools.h>
+#ifndef DISABLE_GUI
+#ifndef GNUSTEP
 #import <simtoolsgui.h>
+#endif
+#endif
+
+#ifdef ENABLE_XMLRPC
+#import <swarm_xmlrpc.h>
+#endif
 
 #import <objectbase/probing.h> // initProbing
 #import <defobj/Customize.h>  // PHASE
 #import <defobj/Arguments.h> // Arguments_c
 
+#ifndef DISABLE_GUI
+#ifndef GNUSTEP
 #import <gui.h> // GUI_EVENT_ASYNC
+#endif
+#endif
 
 #include <swarmconfig.h>
 
@@ -95,12 +107,12 @@ id (*_swarm_i_Object_s__drop) (struct Object_s *, struct objc_selector *);
 
 static void predispatch ()
 {
-  (IMP) _swarm_i_Zone_c__allocIVarsComponent_ = get_imp (objc_lookup_class ("Zone_c"), M(allocIVarsComponent:));
-  (IMP) _swarm_i_Zone_c__freeIVarsComponent_ = get_imp (objc_lookup_class ("Zone_c"), M(freeIVarsComponent:));
-  (IMP) _swarm_i_Zone_c__allocBlock_ = get_imp (objc_lookup_class ("Zone_c"), M(allocBlock:));
-  (IMP) _swarm_i_Zone_c__freeBlock_blockSize_ = get_imp (objc_lookup_class ("Zone_c"), M(freeBlock:blockSize:));
-  (IMP) _swarm_i_ComponentZone_c__allocIVars_ = get_imp (objc_lookup_class ("ComponentZone_c"), M(allocIVars:));
-  (IMP) _swarm_i_Object_s__drop = get_imp (objc_lookup_class ("Object_s"), M(drop));
+   _swarm_i_Zone_c__allocIVarsComponent_ = get_imp (objc_lookup_class ("Zone_c"), M(allocIVarsComponent:));
+   _swarm_i_Zone_c__freeIVarsComponent_ = get_imp (objc_lookup_class ("Zone_c"), M(freeIVarsComponent:));
+  _swarm_i_Zone_c__allocBlock_ = get_imp (objc_lookup_class ("Zone_c"), M(allocBlock:));
+  _swarm_i_Zone_c__freeBlock_blockSize_ = get_imp (objc_lookup_class ("Zone_c"), M(freeBlock:blockSize:));
+  _swarm_i_ComponentZone_c__allocIVars_ = get_imp (objc_lookup_class ("ComponentZone_c"), M(allocIVars:));
+  _swarm_i_Object_s__drop = get_imp (objc_lookup_class ("Object_s"), M(drop));
 }
 
 
@@ -132,14 +144,22 @@ PHASE(Creating)
 {
   initDefobj (arguments);
   initProbing ();
-  
+
+#ifndef DISABLE_GUI  
   if (![arguments getBatchModeFlag] && !forceBatchMode)
     swarmGUIMode = YES;
-  
+#else
+  swarmGUIMode = NO;
+#endif
+
   initRandom (arguments);
-  
+
+#ifndef DISABLE_GUI  
+#ifndef GNUSTEP
   if (swarmGUIMode)
     initSimtoolsGUI ();
+#endif
+#endif
 
 #ifdef HAVE_JDK
   {
@@ -154,6 +174,10 @@ PHASE(Creating)
       }
   }
 #endif
+ 
+#ifdef ENABLE_XMLRPC 
+  initSwarmXMLRPC();
+#endif
   return self;
 }
 
@@ -216,6 +240,8 @@ PHASE(Using)
   return getCurrentSwarmActivity ();
 }
 
+#ifndef DISABLE_GUI
+#ifndef GNUSTEP
 - (void)createProbeDisplay: obj
 {
   CREATE_PROBE_DISPLAY (obj);
@@ -255,15 +281,17 @@ PHASE(Using)
         name: name];
 }
 
-- (void)xprint: obj
-{
-  xprint (obj);
-}
-
 - (void)updateDisplay
 {
   while (GUI_EVENT_ASYNC ()) {}
 }
+#endif // GNUSTEP
+#endif // DISABLE_GUI
+
+- (void)xprint: obj
+{
+  xprint (obj);
+}
 
 - (void)xfprint: obj
 {
@@ -300,11 +328,17 @@ _initSwarm_ (int argc, const char **argv, const char *appName,
              BOOL inhibitExecutableSearchFlag)
 {
   id env;
+#ifndef GNUSTEP
   void __objc_exec_class_for_all_initial_modules ();
 
   __objc_exec_class_for_all_initial_modules ();
+#endif
   env = [SwarmEnvironment createBegin];
 
+#if DEBUG
+  fprintf(stderr, "Swarm Environment\n");
+  fprintf(stderr, "argc argv[0]: %d %s\n", argc, argv[0]);
+#endif
   [env setArguments:
          [(id) argumentsClass ?: (id) [Arguments_c class]
                          createArgc: argc
