@@ -41,7 +41,13 @@ Library:      defobj
 #include <misc.h> // strcpy, strlen, sprintf, isDigit
 #include <collections/predicates.h> // arrayp, keywordp, archiver_list_p, stringp
 
+#ifdef GNUSTEP
+#include <Foundation/NSMethodSignature.h>
+#else
+#ifdef USE_MFRAME
 #include <objc/mframe.h>
+#endif
+#endif
 
 #ifdef HAVE_JDK
 #include "java.h"
@@ -91,7 +97,7 @@ PHASE(Creating)
   return self;
 }
 
-- hdf5InCreate: hdf5Obj
+- hdf5InCreate: (id <HDF5>)hdf5Obj
 {
   return self;
 }
@@ -119,7 +125,7 @@ PHASE(Setting)
   return self;
 }
 
-- hdf5In: hdf5Obj
+- hdf5In: (id <HDF5>)hdf5Obj
 {
   if ([hdf5Obj getDatasetFlag])
     [hdf5Obj shallowLoadObject: self];
@@ -175,12 +181,12 @@ PHASE(Using)
 //
 // getClass -- get class object that implements behavior of object
 //
-+ getClass
++ (Class)getClass
 {
   return getClass (self);
 }
 
-- getClass
+- (Class)getClass
 {
   return SD_GETCLASS (self);
 }
@@ -612,6 +618,7 @@ _obj_dropAlloc (mapalloc_t mapalloc, BOOL objectAllocation)
   return mptr (self, aSel, anObject1, anObject2, anObject3);
 }
 
+#ifdef USE_MFRAME
 - (retval_t)forward: (SEL)aSel : (arglist_t)argFrame
 {
   NSArgumentInfo info;
@@ -730,6 +737,7 @@ _obj_dropAlloc (mapalloc_t mapalloc, BOOL objectAllocation)
       return NULL;
     }
 }
+#endif
 
 //
 // methods inherited from Object that are blocked because they do not support
