@@ -1,7 +1,21 @@
 // Swarm library. Copyright  1996-2000 Swarm Development Group.
-// This library is distributed without any warranty; without even the
-// implied warranty of merchantability or fitness for a particular purpose.
-// See file LICENSE for details and terms of copying.
+// This program is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
+//
+// This program is distributed in the hope that it will be useful, but
+// WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+// General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with this program; if not, write to the Free Software
+// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+// USA
+// 
+// The Swarm Development Group can be reached via our website at:
+// http://www.swarm.org/
 
 /*
 Name:         FCall.m
@@ -13,6 +27,8 @@ Library:      defobj
 #import <defobj.h>
 #import <defobj/FArguments.h>
 #import <objc/objc-api.h>
+#import <defobj/defalloc.h>
+#import <defobj/macros.h>
 
 #include <swarmconfig.h>
 #ifndef USE_AVCALL
@@ -23,23 +39,41 @@ Library:      defobj
 #undef VERSION
 #else
 #include "fcall_objc.h"
+#ifdef HAVE_JDK
 #include "fcall_java.h"
 #endif
+#endif
 
 #include <misc.h>
+#ifdef GNUSTEP
+#include <Foundation/NSMethodSignature.h>
+// mframe internals not available
+#define BUGGY_BUILTIN_APPLY
+#else
+#ifdef USE_MFRAME
 #include <objc/mframe.h>
+#endif
+#endif
+
+#import <defobj/directory.h>
 
 #ifdef HAVE_JDK
-#include <defobj/directory.h>
+#include "java.h" // swarm_directory_java_ensure_objc
 JNIEnv *jniEnv;
 #include <javavars.h>
 #endif
 
+#import "COM.h"
+
 #ifdef HAVE_JDK
 void * java_static_call_functions[FCALL_TYPE_COUNT];
 void * java_call_functions[FCALL_TYPE_COUNT];
 #endif
 
+#define GCINFO_SIZE (2 * sizeof (void *))
+#define gc_fclass gcInfo[0]
+#define gc_fobject gcInfo[1]
+
 #ifndef HAVE_JDK
 static void
 java_not_available (void)
@@ -97,6 +131,8 @@ defobj_init_java_call_tables (void *jEnv)
       FUNCPTR ((*(JNIEnv *) jEnv)->CallStaticObjectMethod);
   java_static_call_functions[fcall_type_jstring] = 
       FUNCPTR ((*(JNIEnv *) jEnv)->CallStaticObjectMethod);
+  java_static_call_functions[fcall_type_jselector] = 
+      FUNCPTR ((*(JNIEnv *) jEnv)->CallStaticObjectMethod);
 
   java_call_functions[fcall_type_void] = 
       FUNCPTR ((*(JNIEnv *) jEnv)->CallVoidMethod);
@@ -133,6 +169,8 @@ defobj_init_java_call_tables (void *jEnv)
       FUNCPTR ((*(JNIEnv *) jEnv)->CallObjectMethod);
   java_call_functions[fcall_type_jstring] = 
       FUNCPTR ((*(JNIEnv *) jEnv)->CallObjectMethod);
+  java_call_functions[fcall_type_jselector] = 
+      FUNCPTR ((*(JNIEnv *) jEnv)->CallObjectMethod);
 }
 #endif
 
@@ -145,13 +183,16 @@ fillHiddenArguments (FCall_c *self)
     {
     case ccall:
       break;
+    case COMcall:
+    case JScall:
+      break;
     case objccall: 
       fargs->hiddenArgumentCount = 2;	
 #ifdef USE_AVCALL
-      objc_setup_call (fargs, self->fobject, self->fmethod);
+      objc_setup_call (fargs, self->gc_fobject, self->fmethod);
 #else
       fargs->ffiArgTypes[MAX_HIDDEN - 2] = &ffi_type_pointer;
-      fargs->argValues[MAX_HIDDEN - 2] = &self->fobject;
+      fargs->argValues[MAX_HIDDEN - 2] = &self->gc_fobject;
       fargs->ffiArgTypes[MAX_HIDDEN - 1] = &ffi_type_pointer;
       fargs->argValues[MAX_HIDDEN - 1] = &self->fmethod;
 #endif
@@ -160,12 +201,12 @@ fillHiddenArguments (FCall_c *self)
     case javacall:
       fargs->hiddenArgumentCount = 3;
 #ifdef USE_AVCALL
-      java_setup_call (fargs, jniEnv, self->fobject, self->fmethod);
+      java_setup_call (fargs, self->gc_fobject, self->fmethod);
 #else
       fargs->ffiArgTypes[MAX_HIDDEN - 3] = &ffi_type_pointer;
       fargs->argValues[MAX_HIDDEN - 3] = &jniEnv;
       fargs->ffiArgTypes[MAX_HIDDEN - 2] = &ffi_type_pointer;
-      fargs->argValues[MAX_HIDDEN - 2] = &self->fobject;
+      fargs->argValues[MAX_HIDDEN - 2] = &self->gc_fobject;
       fargs->ffiArgTypes[MAX_HIDDEN - 1] = &ffi_type_pointer;
       fargs->argValues[MAX_HIDDEN - 1] = &self->fmethod;
 #endif
@@ -173,12 +214,12 @@ fillHiddenArguments (FCall_c *self)
     case javastaticcall:
       fargs->hiddenArgumentCount = 3;
 #ifdef USE_AVCALL
-      java_setup_static_call (fargs, jniEnv, self->fclass, self->fmethod);
+      java_setup_static_call (fargs, self->gc_fclass, self->fmethod);
 #else
       fargs->ffiArgTypes[MAX_HIDDEN - 3] = &ffi_type_pointer;
       fargs->argValues[MAX_HIDDEN - 3] = &jniEnv;
       fargs->ffiArgTypes[MAX_HIDDEN - 2] = &ffi_type_pointer;
-      fargs->argValues[MAX_HIDDEN - 2] = &self->fclass;
+      fargs->argValues[MAX_HIDDEN - 2] = &self->gc_fclass;
       fargs->ffiArgTypes[MAX_HIDDEN - 1] = &ffi_type_pointer;
       fargs->argValues[MAX_HIDDEN - 1] = &self->fmethod;
 #endif
@@ -210,7 +251,10 @@ static ffi_type *ffi_types[FCALL_TYPE_COUNT] = {
   &ffi_type_pointer, 
   &ffi_type_pointer,
   &ffi_type_pointer,
-  &ffi_type_pointer};
+  &ffi_type_pointer,
+  &ffi_type_pointer,
+  &ffi_type_pointer 
+};
 #endif
 
 static void
@@ -219,44 +263,75 @@ add_ffi_types (FCall_c *fc)
   unsigned i;
   FArguments_c *fa = fc->fargs;
 
-#ifndef USE_AVCALL
-  fa->ffiReturnType = ffi_types[fa->returnType];
-  fillHiddenArguments (fc);
-  for (i = 0; i < fa->assignedArgumentCount; i++)
+  if (fc->callType == COMcall)
     {
-      unsigned pos = i + MAX_HIDDEN;
-      fcall_type_t type = fa->argTypes[pos];
+      fc->COM_params = COM_create_params (fa->assignedArgumentCount + 1);
       
-      fa->ffiArgTypes[pos] = ffi_types[type];
-    }
-#else
-#ifdef HAVE_JDK
-  if (fc->callType == javacall || fc->callType == javastaticcall)
-    {
-      java_set_return_type (fc, fa);
-      fillHiddenArguments (fc);
       for (i = 0; i < fa->assignedArgumentCount; i++)
         {
           unsigned pos = i + MAX_HIDDEN;
+          val_t val = { fa->argTypes[pos], *(types_t *) fa->argValues[pos] };
           
-          java_add_primitive (fa, fa->argTypes[pos], fa->argValues[pos]);
+          COM_set_arg (fc->COM_params, i, &val);
+        }
+      COM_set_return (fc->COM_params,
+                      fa->assignedArgumentCount,
+                      &fa->retVal);
+    }
+  else if (fc->callType == JScall)
+    {
+      fc->COM_params = JS_create_params (fa->assignedArgumentCount + 1);
+      
+      for (i = 0; i < fa->assignedArgumentCount; i++)
+        {
+          unsigned pos = i + MAX_HIDDEN;
+          val_t val = { fa->argTypes[pos], *(types_t *) fa->argValues[pos] };
+          
+          JS_set_arg (fc->COM_params, i, &val);
         }
     }
   else
-#endif
     {
-      objc_set_return_type (fc, fa);
+#ifndef USE_AVCALL
+      fa->ffiReturnType = ffi_types[fa->retVal.type];
       fillHiddenArguments (fc);
       for (i = 0; i < fa->assignedArgumentCount; i++)
         {
           unsigned pos = i + MAX_HIDDEN;
+          fcall_type_t type = fa->argTypes[pos];
           
-          objc_add_primitive (fa, fa->argTypes[pos], fa->argValues[pos]);
+          fa->ffiArgTypes[pos] = ffi_types[type];
         }
-    }
+#else
+#ifdef HAVE_JDK
+      if (fc->callType == javacall || fc->callType == javastaticcall)
+        {
+          java_set_return_type (fc);
+          fillHiddenArguments (fc);
+          for (i = 0; i < fa->assignedArgumentCount; i++)
+            {
+              unsigned pos = i + MAX_HIDDEN;
+              
+              java_add_primitive (fa, fa->argTypes[pos], fa->argValues[pos]);
+            }
+        }
+      else
 #endif
+        {
+          objc_set_return_type (fc);
+          fillHiddenArguments (fc);
+          for (i = 0; i < fa->assignedArgumentCount; i++)
+            {
+              unsigned pos = i + MAX_HIDDEN;
+              
+              objc_add_primitive (fa, fa->argTypes[pos], fa->argValues[pos]);
+            }
+        }
+#endif
+    }
 }
 
+#define UPDATEMETHODNAME(value) { if (methodName) FREEBLOCK (methodName); methodName = STRDUP (value); }
 
 @implementation FCall_c
 
@@ -265,6 +340,12 @@ PHASE(Creating)
 {
   FCall_c *newCall = [aZone allocIVars: self];
   newCall->fargs = NULL;
+  {
+    JOBJECT *ptr = [_obj_GCFixedRootZone allocBlock: GCINFO_SIZE];
+    
+    memset (ptr, 0, GCINFO_SIZE);
+    newCall->gcInfo = ptr;
+  }
   return newCall;
 }
 
@@ -281,52 +362,146 @@ PHASE(Creating)
   return self;
 }
 
-- setMethod: (SEL)mtd inObject: obj
-{
-  Class cl;
-
-  callType = objccall;
-  (id) fobject = obj;
-  (SEL) fmethod = mtd;
-  cl = getClass (obj);
-  (Class) fclass = cl;
-  ffunction = FUNCPTR (get_imp ((Class) fclass, (SEL) fmethod));
-  return self;
-}
-
-- setJavaMethod: (const char *)mtdName inObject: (JOBJECT)obj
+- _setJavaMethod_: (const char *)theMethodName inObject: (JOBJECT)jObj
 {
 #ifdef HAVE_JDK
   jclass lref;
+  char buf[strlen (theMethodName) + 1], *p;
+
   callType = javacall;
-  if (fclass)
-    (*jniEnv)->DeleteGlobalRef (jniEnv, fclass);
-  lref = (*jniEnv)->GetObjectClass (jniEnv, (jobject) obj);
-  fclass = (*jniEnv)->NewGlobalRef (jniEnv, lref);
+
+  
+  strcpy (buf, theMethodName);
+  p = strchr (buf, ':');
+  if (p)
+    *p = '\0';
+  
+  if (gc_fclass)
+    (*jniEnv)->DeleteGlobalRef (jniEnv, gc_fclass);
+  lref = (*jniEnv)->GetObjectClass (jniEnv, (jobject) jObj);
+  gc_fclass = (*jniEnv)->NewGlobalRef (jniEnv, lref);
   (*jniEnv)->DeleteLocalRef (jniEnv, lref);
-  methodName = STRDUP (mtdName);
-  (jobject) fobject = (*jniEnv)->NewGlobalRef (jniEnv, (jobject) obj);
-  fobjectPendingGlobalRefFlag = YES;
+  UPDATEMETHODNAME (buf);
+  gc_fobject = jObj;
 #else
   java_not_available ();
 #endif
   return self;
 }    
-   
-- setJavaMethod: (const char *)mtdName inClass: (const char *)className
+
+- setMethodFromName: (const char *)theMethodName inObject: obj
+{
+  COMobject cObj = SD_COM_FIND_OBJECT_COM (obj);
+
+  if (cObj)
+    {
+      if (COM_is_javascript (cObj))
+        {
+          gc_fobject = SD_COM_FIND_OBJECT_COM (obj);
+          callType = JScall;
+	  UPDATEMETHODNAME (theMethodName);
+          return self;
+        }
+    }
+  else 
+    {
+      SEL sel = sel_get_any_typed_uid (theMethodName);
+
+#ifdef HAVE_JDK
+      if (!sel)
+        {
+          jobject jObj = SD_JAVA_FIND_OBJECT_JAVA (obj);
+          
+          if (jObj)
+            {
+              [self setJavaMethodFromName: theMethodName inObject: jObj];
+              return self;
+            }
+        }
+#endif
+      [self setMethodFromSelector: sel inObject: obj];
+    }
+  return self;
+}
+
+- setMethodFromSelector: (SEL)sel inObject: obj
+{
+  COMselector cSel = SD_COM_FIND_SELECTOR_COM (sel);
+  
+  if (cSel)
+    {
+      callType = COM_selector_is_javascript (cSel) ? JScall : COMcall;
+       fmethod = COM_selector_method (cSel);
+       gc_fobject = SD_COM_FIND_OBJECT_COM (obj);
+      
+      if (callType == JScall)
+        UPDATEMETHODNAME (sel_get_name (sel));
+      return self;
+    }
+#ifdef HAVE_JDK
+  else if ([fargs getLanguage] == LanguageJava)
+    {
+      jobject jsel = SD_JAVA_FIND_SELECTOR_JAVA (sel);
+
+      if (jsel)
+        {
+          jstring string;
+          const char *javaMethodName;
+          jboolean copy;
+          jobject jobj = SD_JAVA_FIND_OBJECT_JAVA (obj);
+          
+          string = (*jniEnv)->GetObjectField (jniEnv, jsel, f_nameFid);
+          javaMethodName =
+            (*jniEnv)->GetStringUTFChars (jniEnv, string, &copy);
+          
+          [(id) self setJavaMethodFromName: javaMethodName inObject: jobj];
+          if (copy)
+            (*jniEnv)->ReleaseStringUTFChars (jniEnv, string,
+                                              javaMethodName);
+          (*jniEnv)->DeleteLocalRef (jniEnv, string);
+          return self;
+        }
+    }
+#endif
+  {
+    Class class;
+    
+    callType = objccall;
+    gc_fobject = obj;
+     fmethod = sel;
+    class = getClass (obj);
+    gc_fclass = class;
+    ffunction = FUNCPTR (get_imp ((Class) gc_fclass, (SEL) fmethod));
+    return self;
+  }
+}
+
+- setJavaMethodFromName: (const char *)theMethodName inObject: (JOBJECT)jObj
+{
+#ifdef HAVE_JDK
+  [self _setJavaMethod_: theMethodName
+        inObject: (*jniEnv)->NewGlobalRef (jniEnv, jObj)];
+  fobjectPendingGlobalRefFlag = YES;
+#else
+  java_not_available ();
+#endif
+  return self;
+}
+
+- setJavaMethodFromName: (const char *)theMethodName inClass: (const char *)className
 { 
 #ifdef HAVE_JDK
   jclass lref;
 
   callType = javastaticcall;
-  if (fclass)
-    (*jniEnv)->DeleteGlobalRef (jniEnv, fclass);
+  if (gc_fclass)
+    (*jniEnv)->DeleteGlobalRef (jniEnv, gc_fclass);
   lref = (*jniEnv)->FindClass (jniEnv, className);
-  fclass = (*jniEnv)->NewGlobalRef (jniEnv, lref);
+  gc_fclass = (*jniEnv)->NewGlobalRef (jniEnv, lref);
   (*jniEnv)->DeleteLocalRef (jniEnv, lref);
-  methodName = STRDUP (mtdName);
+  UPDATEMETHODNAME (theMethodName);
 #else
-  java_not_available();
+  java_not_available ();
 #endif
   return self;
 }
@@ -339,40 +514,52 @@ PHASE(Creating)
     raiseEvent (SourceMessage, "Arguments and return type not specified!\n");
 #ifdef HAVE_JDK
   if (callType == javacall || callType == javastaticcall)
-      {
-        ffunction = (callType == javacall ? 
-                     java_call_functions[fargs->returnType] :
-                     java_static_call_functions[fargs->returnType]);
-        
-        (jmethodID) fmethod =
-          (callType == javacall ?
-           (*jniEnv)->GetMethodID (jniEnv, fclass, 
-                                   methodName, 
-                                   fargs->javaSignature) :
-           (*jniEnv)->GetStaticMethodID (jniEnv, fclass, 
-                                         methodName, 
-                                         fargs->javaSignature)); 
-        if (!fmethod)
-          raiseEvent (SourceMessage, "Could not find Java method: `%s' (%s)\n",
-                      methodName, fargs->javaSignature);
-      }
+    {      
+      (*jniEnv)->ExceptionClear (jniEnv);
+
+      if (callType == javacall)
+        {
+          fmethod =
+            (*jniEnv)->GetMethodID (jniEnv,
+                                    gc_fclass,
+                                    methodName, 
+                                    fargs->javaSignature);
+          ffunction = java_call_functions[fargs->retVal.type];
+        }
+      else
+        {
+          fmethod = (*jniEnv)->GetStaticMethodID (jniEnv,
+                                                  gc_fclass,
+                                                  methodName, 
+                                                  fargs->javaSignature);
+          ffunction = java_static_call_functions[fargs->retVal.type];
+        }
+      if (!fmethod)
+        {
+          (*jniEnv)->ExceptionClear (jniEnv);
+          [self drop];
+          return nil;
+        }
+    }
 #endif
   add_ffi_types (self);
 #ifndef USE_AVCALL
-  {
-    unsigned res;
-    
-    res = ffi_prep_cif (&cif, FFI_DEFAULT_ABI, 
-                        (fargs->hiddenArgumentCount +
-                         fargs->assignedArgumentCount),
-                        (ffi_type *) fargs->ffiReturnType, 
-                        (ffi_type **) fargs->ffiArgTypes + MAX_HIDDEN - 
-                        fargs->hiddenArgumentCount);
-    if (_obj_debug && res != FFI_OK)
-      raiseEvent (SourceMessage,
-                  "Failed while preparing foreign function call closure!\n"); 
-  }
+  if (callType != COMcall && callType != JScall)
+    {
+      unsigned res;
+      
+      res = ffi_prep_cif (&cif, FFI_DEFAULT_ABI, 
+                          (fargs->hiddenArgumentCount +
+                           fargs->assignedArgumentCount),
+                          (ffi_type *) fargs->ffiReturnType, 
+                          (ffi_type **) fargs->ffiArgTypes + MAX_HIDDEN - 
+                          fargs->hiddenArgumentCount);
+      if (_obj_debug && res != FFI_OK)
+        raiseEvent (SourceMessage,
+                    "Failed while preparing foreign function call\n"); 
+    }
 #endif
+  setMappedAlloc (self);
   setNextPhase (self);
   return self;
 }
@@ -384,26 +571,19 @@ PHASE(Creating)
   id <FCall> fc = [FCall createBegin: aZone];
 
   [fc setArguments: fa];
-#ifdef HAVE_JDK
-  if ([fa getJavaFlag])
-    {
-      jstring string;
-      const char *javaMethodName;
-      jboolean copy;
-      jobject jsel = SD_FINDJAVA (jniEnv, (id) aSel);
-      jobject jobj = SD_FINDJAVA (jniEnv, target);
-      
-      string = (*jniEnv)->GetObjectField (jniEnv, jsel, f_nameFid);
-      javaMethodName = (*jniEnv)->GetStringUTFChars (jniEnv, string, &copy);
+  [fc setMethodFromSelector: aSel inObject: target];
 
-      [fc setJavaMethod: javaMethodName inObject: jobj];
-      if (copy)
-        (*jniEnv)->ReleaseStringUTFChars (jniEnv, string, javaMethodName);
-      (*jniEnv)->DeleteLocalRef (jniEnv, string);
-    }
-  else
-#endif
-    [fc setMethod: aSel inObject: target];
+  return [fc createEnd];
+}
+
++ create: aZone target: target
+            methodName: (const char *)theMethodName
+             arguments: (id <FArguments>)fa
+{
+  id <FCall> fc = [FCall createBegin: aZone];
+
+  [fc setArguments: fa];
+  [fc setMethodFromName: theMethodName inObject: target];
 
   return [fc createEnd];
 }
@@ -411,13 +591,26 @@ PHASE(Creating)
 void
 updateTarget (FCall_c *self, id target)
 {
+  if (self->fargs->language == LanguageObjc)
+    {
+      self->gc_fobject = target;
+      add_ffi_types (self);
+    }
 #ifdef HAVE_JDK
-  if ([target respondsTo: M(isJavaProxy)])
-    updateJavaTarget (self, SD_FINDJAVA (jniEnv, target));
-  else
+  else if (self->fargs->language == LanguageJava)
+    {
+      jobject jObj = SD_JAVA_FIND_OBJECT_JAVA (target);
+
+      updateJavaTarget (self, jObj);
+    }
 #endif
-    self->fobject = target;
-  add_ffi_types (self);
+  else if (self->fargs->language == LanguageCOM)
+    {
+      COMobject cObj = SD_COM_FIND_OBJECT_COM (target);
+
+       self->gc_fobject = cObj;
+      add_ffi_types (self);
+    }
 }
 
 #ifdef HAVE_JDK
@@ -426,10 +619,10 @@ updateJavaTarget (FCall_c *self, JOBJECT target)
 {
   if (self->fobjectPendingGlobalRefFlag)
     {
-      (*jniEnv)->DeleteGlobalRef (jniEnv, (jobject) self->fobject);
+      (*jniEnv)->DeleteGlobalRef (jniEnv, (jobject) self->gc_fobject);
       self->fobjectPendingGlobalRefFlag = NO;
     }
-  self->fobject = target;
+  self->gc_fobject = target;
   add_ffi_types (self);
 }
 #endif
@@ -451,115 +644,141 @@ PHASE(Using)
 #ifdef HAVE_JDK
   if (fargs->pendingGlobalRefFlag)
     {
-      (*jniEnv)->DeleteGlobalRef (jniEnv, (jobject) fargs->resultVal.object);
+      (*jniEnv)->DeleteGlobalRef (jniEnv,
+                                  (jobject) fargs->retVal.val.object);
       fargs->pendingGlobalRefFlag = NO;
     }
 #endif
+  if (callType == COMcall)
+    COM_method_invoke ((COMmethod) fmethod,
+                       (COMobject) gc_fobject,
+                       COM_params);
+  else if (callType == JScall)
+    {
+      JS_method_invoke ((COMobject) gc_fobject,
+                        methodName,
+                        COM_params);
+      JS_set_return (COM_params,
+                     fargs->assignedArgumentCount,
+                     &fargs->retVal);
+    }
 #ifndef USE_AVCALL
-  {
-    types_t ret;
-    
-    ffi_call (&cif, ffunction, &ret, fargs->argValues + 
-              MAX_HIDDEN - fargs->hiddenArgumentCount);  
+  else
+    {
+      types_t ret;
+      
+      ffi_call (&cif, ffunction, &ret, fargs->argValues + 
+                MAX_HIDDEN - fargs->hiddenArgumentCount);  
 #ifdef __mips64
 #define VAL(type, var) (*((type *)(((void *)&var)+(sizeof(var)-sizeof(type)))))
 #else
 #define VAL(type, var) (*((type *)&var))
 #endif
-
-  switch (fargs->returnType)
-      {
-      case fcall_type_void:
-        break;
-      case fcall_type_boolean:
-        fargs->resultVal.boolean = VAL (BOOL, ret);
-        break;
-      case fcall_type_schar:
-        // character return is broken in libffi-1.18
-        fargs->resultVal.schar = VAL (int, ret);
-        break;
-      case fcall_type_uchar:
-        // character return is broken in libffi-1.18
-        fargs->resultVal.uchar = VAL (unsigned, ret);
-        break;
-      case fcall_type_sint:
-        fargs->resultVal.sint = VAL (int, ret);
-        break;
-      case fcall_type_uint:
-        fargs->resultVal.uint = VAL (unsigned, ret);
-        break;
-      case fcall_type_sshort:
-        // short return is broken in libffi-1.18
-        fargs->resultVal.sshort = VAL (int, ret);
-        break;
-      case fcall_type_ushort:
-        // short return is broken in libffi-1.18
-        fargs->resultVal.ushort = VAL (unsigned, ret);
-        break;
-      case fcall_type_slong:
-        fargs->resultVal.slong = VAL (long, ret);
-        break;
-      case fcall_type_ulong:
-        fargs->resultVal.ulong = VAL (unsigned long, ret);
-        break;
-      case fcall_type_slonglong:
-        fargs->resultVal.slonglong = VAL (long long, ret);
-        break;
-      case fcall_type_ulonglong:
-        fargs->resultVal.ulonglong = VAL (unsigned long long, ret);
-        break;
-      case fcall_type_float:
-        fargs->resultVal._float = ret._float;
-        break;
-      case fcall_type_double:
-        fargs->resultVal._double = ret._double;
-        break;
-      case fcall_type_long_double:
-        fargs->resultVal._long_double = ret._long_double;
-        break;
-      case fcall_type_object:
-        fargs->resultVal.object = VAL (id, ret);
-        break;
-      case fcall_type_class:
-        fargs->resultVal.class = VAL (Class, ret);
-        break;
-      case fcall_type_selector:
-        fargs->resultVal.selector = VAL (SEL, ret);
-        break;
-      case fcall_type_string:
-        fargs->resultVal.string = VAL (const char *, ret);
-        break;
+      
+      switch (fargs->retVal.type)
+        {
+        case fcall_type_void:
+          break;
+        case fcall_type_boolean:
+          fargs->retVal.val.boolean = VAL (BOOL, ret);
+          break;
+        case fcall_type_schar:
+          // character return is broken in libffi-1.18
+          fargs->retVal.val.schar = VAL (int, ret);
+          break;
+        case fcall_type_uchar:
+          // character return is broken in libffi-1.18
+          fargs->retVal.val.uchar = VAL (unsigned, ret);
+          break;
+        case fcall_type_sint:
+          fargs->retVal.val.sint = VAL (int, ret);
+          break;
+        case fcall_type_uint:
+          fargs->retVal.val.uint = VAL (unsigned, ret);
+          break;
+        case fcall_type_sshort:
+          // short return is broken in libffi-1.18
+          fargs->retVal.val.sshort = VAL (int, ret);
+          break;
+        case fcall_type_ushort:
+          // short return is broken in libffi-1.18
+          fargs->retVal.val.ushort = VAL (unsigned, ret);
+          break;
+        case fcall_type_slong:
+          fargs->retVal.val.slong = VAL (long, ret);
+          break;
+        case fcall_type_ulong:
+          fargs->retVal.val.ulong = VAL (unsigned long, ret);
+          break;
+        case fcall_type_slonglong:
+          fargs->retVal.val.slonglong = VAL (long long, ret);
+          break;
+        case fcall_type_ulonglong:
+          fargs->retVal.val.ulonglong = VAL (unsigned long long, ret);
+          break;
+        case fcall_type_float:
+          fargs->retVal.val._float = ret._float;
+          break;
+        case fcall_type_double:
+          fargs->retVal.val._double = ret._double;
+          break;
+        case fcall_type_long_double:
+          fargs->retVal.val._long_double = ret._long_double;
+          break;
+        case fcall_type_object:
+          fargs->retVal.val.object = VAL (id, ret);
+          break;
+        case fcall_type_class:
+          fargs->retVal.val._class = VAL (Class, ret);
+          break;
+        case fcall_type_selector:
+          fargs->retVal.val.selector = VAL (SEL, ret);
+          break;
+        case fcall_type_string:
+          fargs->retVal.val.string = VAL (const char *, ret);
+          break;
 #ifdef HAVE_JDK
-      case fcall_type_jobject:
-	fargs->resultVal.object = (id) VAL (jobject, ret);
-	break;
-      case fcall_type_jstring:
-	fargs->resultVal.object = (id) VAL (jstring, ret);
-	break;
+        case fcall_type_jobject:
+          fargs->retVal.val.object = (id) VAL (jobject, ret);
+          break;
+        case fcall_type_jstring:
+          fargs->retVal.val.object = (id) VAL (jstring, ret);
+          break;
 #endif
-      default:
-        abort ();
-      }
-  }
+        default:
+          abort ();
+        }
+    }
 #else
 #ifdef HAVE_JDK
-  if (callType == javacall || callType == javastaticcall)
+  else if (callType == javacall || callType == javastaticcall)
     java_call (fargs);
-  else
+  else 
 #endif
     objc_call (fargs);
 #endif
 #ifdef HAVE_JDK
   if (callType == javacall || callType == javastaticcall)
     {
-      if (fargs->returnType == fcall_type_jobject
-	  || fargs->returnType == fcall_type_jstring)
+      jobject exception;
+
+      if ((exception = (*jniEnv)->ExceptionOccurred (jniEnv)))
+        {
+          (*jniEnv)->ExceptionDescribe (jniEnv);
+          (*jniEnv)->ExceptionClear (jniEnv);
+          (*jniEnv)->DeleteLocalRef (jniEnv, exception);
+
+          raiseEvent (InvalidOperation,
+                      "Exception occurred in Java method `%s'\n", methodName);
+        }
+      if (fargs->retVal.type == fcall_type_jobject
+	  || fargs->retVal.type == fcall_type_jstring)
 	{
-	  jobject lref = (jobject) fargs->resultVal.object;
+	  jobject lref = (jobject) fargs->retVal.val.object;
 	  
-	  fargs->resultVal.object =
+	  fargs->retVal.val.object =
 	    (id) (*jniEnv)->NewGlobalRef (jniEnv, lref);
-	  (*jniEnv)->DeleteLocalRef (jniEnv, lref);
+          (*jniEnv)->DeleteLocalRef (jniEnv, lref);
 	  fargs->pendingGlobalRefFlag = YES;
 	}
     }
@@ -576,23 +795,24 @@ PHASE(Using)
 #ifdef HAVE_JDK
   id return_jobject (void)
     {
-      return SD_ENSUREOBJC (jniEnv, (jobject) buf->object);
+      return SD_JAVA_ENSURE_OBJECT_OBJC ((jobject) buf->object);
     }
   const char *return_jstring (void)
     {
-      const char *newString =
-        swarm_directory_copy_java_string (jniEnv, (jstring) buf->object);
+      const char *newString = JAVA_COPY_STRING ((jstring) buf->object);
       
-      (void) SD_SWITCHOBJC (jniEnv, (jstring) buf->object, (id) newString);
+      (void) SD_JAVA_SWITCHOBJC ((jstring) buf->object,
+                                 (id) newString);
       return newString;
     }
 #endif
 
-#if defined(__i386__) && (__GNUC__ == 2) && (__GNUC_MINOR__ < 95)
+#if (defined(__i386__) && ((__GNUC__ == 2) && ((__GNUC_MINOR__ < 95)  || (__GNUC_MINOR__ == 96))) || (__GNUC__ == 3 && defined(__i386__)) || defined (__CYGWIN__))
+// Let's see how things go now with >=  3.4 - mgd
 #define BUGGY_BUILTIN_APPLY
 #endif
 
-  *buf = ((FArguments_c *) fargs)->resultVal;
+  *buf = ((FArguments_c *) fargs)->retVal.val;
 
 #ifndef BUGGY_BUILTIN_APPLY
   {
@@ -612,7 +832,7 @@ PHASE(Using)
     double return_double (void) { return buf->_double; }
     long double return_long_double (void) { return buf->_long_double; }
     id return_object (void) { return buf->object; }
-    Class return_class (void) { return buf->class; }
+    Class return_class (void) { return buf->_class; }
     void return_void (void) { return; }
     void apply (apply_t func)
       {
@@ -622,7 +842,7 @@ PHASE(Using)
       }
 #define APPLY(func) apply ((apply_t) func)
     
-    switch (fargs->returnType)
+    switch (fargs->retVal.type)
       {
       case fcall_type_void:
         APPLY (return_void);
@@ -695,7 +915,7 @@ PHASE(Using)
   {
     void *ptr;
     
-    switch (fargs->returnType)
+    switch (fargs->retVal.type)
       {
       case fcall_type_void:
         ptr = NULL;
@@ -773,7 +993,7 @@ PHASE(Using)
         ptr = &buf->object;
         break;
       case fcall_type_class:
-        ptr = &buf->class;
+        ptr = &buf->_class;
         break;
       case fcall_type_string:
         ptr = &buf->string;
@@ -804,21 +1024,35 @@ PHASE(Using)
     abort ();
 }
 
+- (void)mapAllocations: (mapalloc_t)mapalloc
+{
+}
+
 - (void)dropAllocations: (BOOL)componentAlloc
 {
+  if (callType == COMcall)
+    COM_free_params (COM_params);
+  else if (callType == JScall)
+    JS_free_params (COM_params);
 #ifdef HAVE_JDK
-  if (callType == javacall)
+  else if (callType == javacall)
     {
       if (fobjectPendingGlobalRefFlag)
-        (*jniEnv)->DeleteGlobalRef (jniEnv, fobject);
-      (*jniEnv)->DeleteGlobalRef (jniEnv, fclass);
+        (*jniEnv)->DeleteGlobalRef (jniEnv, gc_fobject);
+      (*jniEnv)->DeleteGlobalRef (jniEnv, gc_fclass);
     }
   else if (callType == javastaticcall)
-    (*jniEnv)->DeleteGlobalRef (jniEnv, fclass);
+    (*jniEnv)->DeleteGlobalRef (jniEnv, gc_fclass);
 #endif
+  [_obj_GCFixedRootZone freeBlock: gcInfo blockSize: GCINFO_SIZE];
+
   if (methodName)
     FREEBLOCK (methodName);
   [super dropAllocations: componentAlloc];
 }
 
+- (void)drop
+{
+  [self dropAllocations: YES];
+}
 @end
