@@ -17,6 +17,7 @@
 // The Swarm Development Group can be reached via our website at:
 // http://www.swarm.org/
 
+#define H5_USE_16_API
 #include <swarmconfig.h> // HAVE_HDF5, PTRINT, HAVE_JDK
 
 #import <defobj/HDF5Object.h>
@@ -744,7 +745,10 @@ PHASE(Using)
               
               for (i = 0; i < rank; i++)
                 count *= dims[i];
-              memcpy (val_ptr, buf + hoffset, fcall_type_size (type) * count);
+              memcpy (val_ptr, buf + hoffset, fcall_type_size (type) * count); 
+              if (H5Tconvert (mtid, tid_for_fcall_type (type),
+                              count, val_ptr, NULL, H5P_DEFAULT) < 0)
+                raiseEvent (LoadError, "cannot convert member to native");
             }
         }
       inum++;
@@ -813,13 +817,17 @@ PHASE(Using)
         }
       else
         {
+          unsigned count = 1, i;
+
           if (!compare_types (ivarType, type))
             raiseEvent (SaveError,
                         "differing source and target types %d/%d (%s)\n",
                         ivarType, type, ivarName);
+          for (i = 0; i < rank; i++)
+            count *= dims[i];
           memcpy (buf + hoffset,
                   val_ptr,
-                  fcall_type_size (type));
+                  fcall_type_size (type) * count);
         }
       inum++;
     }
@@ -1038,7 +1046,7 @@ PHASE(Creating)
   return self;
 }
 
-#if ((H5_VERS_MAJOR > 1) || (H5_VERS_MAJOR == 1 && H5_VERS_MINOR >= 4))
+#if ((H5_VERS_MAJOR > 1) || (H5_VERS_MAJOR == 1 && H5_VERS_MINOR >= 4)) && 0
 #define count_size_t hsize_t
 #else
 #define count_size_t size_t
@@ -1081,7 +1089,7 @@ string_ref (hid_t sid, hid_t did, H5T_cdata_t *cdata,
   else if (cdata->command == H5T_CONV_CONV)
     {
       size_t size = H5Tget_size (sid);
-      unsigned char *srcptr = buf;
+      char *srcptr = buf;
       size_t i;
 
       for (i = 0; i < count; i++)
@@ -1558,7 +1566,7 @@ PHASE(Using)
 #endif
 }
 
-- (void)iterate: (int (*) (id hdf5obj))iterateFunc drop: (BOOL)dropFlag
+- (void)iterate: (int (*) (id <HDF5> hdf5obj))iterateFunc drop: (BOOL)dropFlag
 {
 #ifdef HAVE_HDF5
   herr_t process_object (hid_t oid, const char *memberName, void *client)
@@ -1616,12 +1624,11 @@ PHASE(Using)
 #endif  
 }
 
-- (void)iterate: (int (*) (id hdf5obj))iterateFunc
+- (void)iterate: (int (*) (id <HDF5> hdf5obj))iterateFunc
 {
   [self iterate: iterateFunc drop: YES];
 }
 
-
 - (void)iterateAttributes: (int (*) (const char *key, const char *value))iterateFunc
 {
 #ifdef HAVE_HDF5
@@ -1658,7 +1665,7 @@ PHASE(Using)
 #endif
 }
 
-- getClass
+- (Class)getClass
 {
 #ifdef HAVE_HDF5
   if (datasetFlag)
