@@ -348,8 +348,9 @@ map_object_ivars (id object,
 }
 
 struct objc_ivar *
-find_ivar (Class class, const char *name)
+find_ivar (id obj, const char *name)
 {
+  Class class = (Class)obj;
   struct objc_ivar_list *ivars = class->ivars;
 
   if (class->super_class)
@@ -525,7 +526,14 @@ lisp_output_type (fcall_type_t type,
       abort ();
       break;
     case fcall_type_string:
-      [stream catString: ((const char **) ptr)[offset]];
+      {
+        const char *str = ((const char **) ptr)[offset];
+
+        if (str)
+          [stream catString: str];
+	else
+	  [stream catNil];
+      }
       break;
     case fcall_type_iid:
       [stream catPointer: ((void **) ptr)[offset]];
@@ -618,6 +626,7 @@ lisp_type_for_objc_type (const char *varType,
           baseType = TYPE_LONG_DOUBLE;
           break;
         case _C_CHARPTR:
+        case 'r':
           baseType = TYPE_STRING;
           break;
         case _C_ID:
@@ -761,6 +770,9 @@ fcall_type_for_objc_type (char objcType)
     if (objcType == objc_types[i])
       return (fcall_type_t) i;
 
+  if (objcType == 'r')
+    return fcall_type_string;
+
   // This is only for eventOccuredOn:via:withProbeType:on:ofType:withData:
   if (objcType == _C_PTR)
     return fcall_type_uint;
@@ -1015,8 +1027,8 @@ typeError (const char *ivar_name,
        }}
 
 #define ENSUREVALUETYPE(expr,vMethod,value_type,ivar_type,ivar_name,buf_expr) \
-  if (ivar_type != value_type)                                                \
-    typeError (ivar_name, ivar_type, value_type);                             \
+  if (ivar_type != value_type)                                              \
+    typeError (ivar_name, ivar_type, value_type);                           \
   buf_expr = [expr vMethod];
 
 void
@@ -1081,7 +1093,14 @@ object_setVariableFromExpr (id obj, const char *ivar_name, id expr)
       switch (value_type)
         {
         case fcall_type_object:
-          ENSUREVALUETYPE (expr, getObject, fcall_type_object, ivar_type, ivar_name, buf.object);
+          if (ivar_type == fcall_type_string) {
+            if ([expr getObject] == NULL)
+              buf.object = NULL;
+            else
+              abort ();
+	  } else {
+            ENSUREVALUETYPE (expr, getObject, fcall_type_object, ivar_type, ivar_name, buf.object);
+          }
           break;
         case fcall_type_class:
           ENSUREVALUETYPE (expr, getClass, fcall_type_class, ivar_type, ivar_name, buf._class);
