@@ -5,7 +5,17 @@
 #import <defobj/JavaClassProxy.h>
 #import <defobj/JavaCollection.h>
 #import <defobj/defalloc.h>
-#include <objc/mframe.h> // mframe_build_signature
+#include <misc.h>
+
+// mframe_build_signature
+#ifdef GNUSTEP
+#include <Foundation/NSMethodSignature.h>
+#else
+#ifdef USE_MFRAME
+#include <objc/mframe.h>
+#endif
+#endif
+
 #import "internal.h" // FCALL_TYPE_COUNT, objc_type_for_fcall_type
 
 #define extern
@@ -1525,6 +1535,8 @@ swarm_directory_java_associate_objects (jobject swarmEnvironment)
     ASSOCIATE (uniformDblRand);
   }
 
+#ifndef DISABLE_GUI
+#ifndef GNUSTEP
   if (swarmGUIMode)
     {
       extern id probeDisplayManager;
@@ -1540,6 +1552,8 @@ swarm_directory_java_associate_objects (jobject swarmEnvironment)
       ASSOCIATE (ControlStateQuit);
       ASSOCIATE (ControlStateNextTime);
     }
+#endif
+#endif
   
   {
     extern id <Symbol> Initialized, Running, Stopped, Holding, Released, 
