@@ -68,7 +68,9 @@ typedef struct raster_private {
 #endif
 } raster_private_t;
 
-extern TkExtra *globalTkInterp;
+/*extern TkExtra *globalTkInterp;*/
+extern id <TkExtra>globalTkInterp;
+
 
 Tk_Window 
 tkobjc_nameToWindow (const char *widgetName)
@@ -124,7 +126,11 @@ setSecondaryPath (id arguments)
   
   if (swarmHome)
     {
+#ifdef __MINGW32__
+      const char *secondarySubpath = "lib/";
+#else
       const char *secondarySubpath = "share/";
+#endif
       char *libraryPath = xmalloc (strlen (swarmHome) +
                                    strlen (secondarySubpath) + 1);
       char *p;
@@ -138,8 +144,8 @@ setSecondaryPath (id arguments)
 void
 tkobjc_initTclInterp (id arguments)
 {
-  globalTkInterp = [TclInterp alloc];  // misnomer
-  setSecondaryPath (arguments);
+  globalTkInterp = [TclInterp alloc];  
+//  setSecondaryPath (arguments);
   [globalTkInterp initWithArgc: 1 /* [arguments getArgc] */
                   argv: [arguments getArgv]];
   registerInterp ();
@@ -149,7 +155,7 @@ void
 tkobjc_initTkInterp (id arguments)
 {
   globalTkInterp = [TkExtra alloc];
-  setSecondaryPath (arguments);
+//  setSecondaryPath (arguments);
 
   [globalTkInterp initWithArgc: 1 /* [arguments getArgc] */
                   argv: [arguments getArgv]];
